/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.boat;

import java.io.*;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * AbstractKeyboardFactory is generated by Relaxer based on keyboard.rng.
 *
 * @version keyboard.rng 1.0 (Mon Jan 26 11:51:56 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public abstract class AbstractKeyboardFactory implements IKeyboardFactory {
    protected org.xml.sax.ErrorHandler errorHandler;
    protected org.xml.sax.EntityResolver entityResolver;
    protected String baseUri;

    /**
     * Sets a <b>errorHandler</b>.
     *
     * @param errorHandler
     */
    public void setErrorHandler(org.xml.sax.ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
    }

    /**
     * Gets a <b>errorHandler</b>.
     *
     * @return org.xml.sax.ErrorHandler
     */
    public org.xml.sax.ErrorHandler getErrorHandler() {
        return (errorHandler);
    }

    /**
     * Sets a <b>entityResolver</b>.
     *
     * @param entityResolver
     */
    public void setEntityResolver(org.xml.sax.EntityResolver entityResolver) {
        this.entityResolver = entityResolver;
    }

    /**
     * Gets a <b>entityResolver</b>.
     *
     * @return org.xml.sax.EntityResolver
     */
    public org.xml.sax.EntityResolver getEntityResolver() {
        return (entityResolver);
    }

    /**
     * Sets a <b>baseUri</b>.
     *
     * @param baseUri
     */
    public void setBaseUri(String baseUri) {
        this.baseUri = baseUri;
    }

    /**
     * Gets a <b>baseUri</b>.
     *
     * @return String
     */
    public String getBaseUri() {
        return (baseUri);
    }

    /**
     * Creates a <code>Object</code> by the File <code>file</code>.
     * This method is a template method for concrete classes.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Object
     */
    public Object create(File file) throws IOException, SAXException, ParserConfigurationException {
        Object node = create(UJAXP.getDocument(file, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (node);
    }

    /**
     * Creates a <code>Object</code> by the String representation of URI <code>uri</code>.
     * This method is a template method for concrete classes.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Object
     */
    public Object create(String uri) throws IOException, SAXException, ParserConfigurationException {
        Object node = create(UJAXP.getDocument(uri, baseUri, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (node);
    }

    /**
     * Creates a <code>Object</code> by the URL <code>url</code>.
     * This method is a template method for concrete classes.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Object
     */
    public Object create(URL url) throws IOException, SAXException, ParserConfigurationException {
        Object node = create(UJAXP.getDocument(url, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (node);
    }

    /**
     * Creates a <code>Object</code> by the InputStream <code>in</code>.
     * This method is a template method for concrete classes.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Object
     */
    public Object create(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        Object node = create(UJAXP.getDocument(in, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (node);
    }

    /**
     * Creates a <code>Object</code> by the InputSource <code>is</code>.
     * This method is a template method for concrete classes.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Object
     */
    public Object create(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        Object node = create(UJAXP.getDocument(is, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (node);
    }

    /**
     * Creates a <code>Object</code> by the Reader <code>reader</code>.
     * This method is a template method for concrete classes.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Object
     */
    public Object create(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        Object node = create(UJAXP.getDocument(reader, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (node);
    }

    /**
     * Creates a <code>Object</code> by the copy <code>Object</code>.
     * This method is a template method for concrete classes.
     *
     * @param source
     * @return Object
     */
    public Object create(Object source) {
        if (source instanceof StartKeyboard) {
            StartKeyboard node = createStartKeyboard();
            node.setup((StartKeyboard)source);
            return(node);
        } else if (source instanceof StartKeyboardKey) {
            StartKeyboardKey node = createStartKeyboardKey();
            node.setup((StartKeyboardKey)source);
            return(node);
        } else if (source instanceof SizeSize1) {
            SizeSize1 node = createSizeSize1();
            node.setup((SizeSize1)source);
            return(node);
        } else if (source instanceof Label) {
            Label node = createLabel();
            node.setup((Label)source);
            return(node);
        } else if (source instanceof LabelType1) {
            LabelType1 node = createLabelType1();
            node.setup((LabelType1)source);
            return(node);
        } else {
            return (null);
        }
    }

    /**
     * Creates a <code>Object</code> by the Document <code>doc</code>.
     * This method is a template method for concrete classes.
     *
     * @param doc
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Object
     */
    public Object create(Document doc) throws IOException, SAXException, ParserConfigurationException {
        Object node = create(doc.getDocumentElement());
        return (node);
    }

    /**
     * Creates a <code>Object</code> by the Element <code>element</code>.
     * This method is a template method for concrete classes.
     *
     * @param element
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Object
     */
    public Object create(Element element) throws IOException, SAXException, ParserConfigurationException {
        if (StartKeyboard.isMatch(element)) {
            StartKeyboard node = createStartKeyboard();
            node.setup(element);
            return(node);
        } else if (StartKeyboardKey.isMatch(element)) {
            StartKeyboardKey node = createStartKeyboardKey();
            node.setup(element);
            return(node);
        } else if (SizeSize1.isMatch(element)) {
            SizeSize1 node = createSizeSize1();
            node.setup(element);
            return(node);
        } else if (Label.isMatch(element)) {
            Label node = createLabel();
            node.setup(element);
            return(node);
        } else if (LabelType1.isMatch(element)) {
            LabelType1 node = createLabelType1();
            node.setup(element);
            return(node);
        } else {
            return (null);
        }
    }

    /**
     * Creates a <code>StartKeyboard</code> by the File <code>file</code>.
     * This method is a template method for concrete classes.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(File file) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(UJAXP.getDocument(file, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the File <code>file</code>.
     * This method is a template method for concrete classes.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(File file) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(UJAXP.getDocument(file, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the File <code>file</code>.
     * This method is a template method for concrete classes.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(File file) throws IOException, SAXException, ParserConfigurationException {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(UJAXP.getDocument(file, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the File <code>file</code>.
     * This method is a template method for concrete classes.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Label
     */
    public Label createLabel(File file) throws IOException, SAXException, ParserConfigurationException {
        Label label_ = createLabel();
        label_.setup(UJAXP.getDocument(file, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the File <code>file</code>.
     * This method is a template method for concrete classes.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return LabelType1
     */
    public LabelType1 createLabelType1(File file) throws IOException, SAXException, ParserConfigurationException {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(UJAXP.getDocument(file, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code>
     * by the String representation of URI <code>uri</code>.
     * This method is a template method for concrete classes.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(String uri) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(UJAXP.getDocument(uri, baseUri, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code>
     * by the String representation of URI <code>uri</code>.
     * This method is a template method for concrete classes.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(String uri) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(UJAXP.getDocument(uri, baseUri, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code>
     * by the String representation of URI <code>uri</code>.
     * This method is a template method for concrete classes.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(String uri) throws IOException, SAXException, ParserConfigurationException {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(UJAXP.getDocument(uri, baseUri, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code>
     * by the String representation of URI <code>uri</code>.
     * This method is a template method for concrete classes.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Label
     */
    public Label createLabel(String uri) throws IOException, SAXException, ParserConfigurationException {
        Label label_ = createLabel();
        label_.setup(UJAXP.getDocument(uri, baseUri, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code>
     * by the String representation of URI <code>uri</code>.
     * This method is a template method for concrete classes.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return LabelType1
     */
    public LabelType1 createLabelType1(String uri) throws IOException, SAXException, ParserConfigurationException {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(UJAXP.getDocument(uri, baseUri, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the URL <code>url</code>.
     * This method is a template method for concrete classes.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(URL url) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the URL <code>url</code>.
     * This method is a template method for concrete classes.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(URL url) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the URL <code>url</code>.
     * This method is a template method for concrete classes.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(URL url) throws IOException, SAXException, ParserConfigurationException {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the URL <code>url</code>.
     * This method is a template method for concrete classes.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Label
     */
    public Label createLabel(URL url) throws IOException, SAXException, ParserConfigurationException {
        Label label_ = createLabel();
        label_.setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the URL <code>url</code>.
     * This method is a template method for concrete classes.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return LabelType1
     */
    public LabelType1 createLabelType1(URL url) throws IOException, SAXException, ParserConfigurationException {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the InputStream <code>in</code>.
     * This method is a template method for concrete classes.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the InputStream <code>in</code>.
     * This method is a template method for concrete classes.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the InputStream <code>in</code>.
     * This method is a template method for concrete classes.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the InputStream <code>in</code>.
     * This method is a template method for concrete classes.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Label
     */
    public Label createLabel(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        Label label_ = createLabel();
        label_.setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the InputStream <code>in</code>.
     * This method is a template method for concrete classes.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return LabelType1
     */
    public LabelType1 createLabelType1(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the InputSource <code>is</code>.
     * This method is a template method for concrete classes.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the InputSource <code>is</code>.
     * This method is a template method for concrete classes.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the InputSource <code>is</code>.
     * This method is a template method for concrete classes.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the InputSource <code>is</code>.
     * This method is a template method for concrete classes.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Label
     */
    public Label createLabel(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        Label label_ = createLabel();
        label_.setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the InputSource <code>is</code>.
     * This method is a template method for concrete classes.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return LabelType1
     */
    public LabelType1 createLabelType1(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the Reader <code>reader</code>.
     * This method is a template method for concrete classes.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the Reader <code>reader</code>.
     * This method is a template method for concrete classes.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the Reader <code>reader</code>.
     * This method is a template method for concrete classes.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the Reader <code>reader</code>.
     * This method is a template method for concrete classes.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return Label
     */
    public Label createLabel(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        Label label_ = createLabel();
        label_.setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the Reader <code>reader</code>.
     * This method is a template method for concrete classes.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     * @return LabelType1
     */
    public LabelType1 createLabelType1(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE, errorHandler, entityResolver));
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the object <code>source</code>.
     * This method is a template method for concrete classes.
     *
     * @param source
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(StartKeyboard source) {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(source);
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the object <code>source</code>.
     * This method is a template method for concrete classes.
     *
     * @param source
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(StartKeyboardKey source) {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(source);
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the object <code>source</code>.
     * This method is a template method for concrete classes.
     *
     * @param source
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(SizeSize1 source) {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(source);
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the object <code>source</code>.
     * This method is a template method for concrete classes.
     *
     * @param source
     * @return Label
     */
    public Label createLabel(Label source) {
        Label label_ = createLabel();
        label_.setup(source);
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the object <code>source</code>.
     * This method is a template method for concrete classes.
     *
     * @param source
     * @return LabelType1
     */
    public LabelType1 createLabelType1(LabelType1 source) {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(source);
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the Document <code>doc</code>.
     * This method is a template method for concrete classes.
     *
     * @param doc
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(Document doc) {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(doc);
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the Document <code>doc</code>.
     * This method is a template method for concrete classes.
     *
     * @param doc
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(Document doc) {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(doc);
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the Document <code>doc</code>.
     * This method is a template method for concrete classes.
     *
     * @param doc
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(Document doc) {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(doc);
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the Document <code>doc</code>.
     * This method is a template method for concrete classes.
     *
     * @param doc
     * @return Label
     */
    public Label createLabel(Document doc) {
        Label label_ = createLabel();
        label_.setup(doc);
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the Document <code>doc</code>.
     * This method is a template method for concrete classes.
     *
     * @param doc
     * @return LabelType1
     */
    public LabelType1 createLabelType1(Document doc) {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(doc);
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the Element <code>element</code>.
     * This method is a template method for concrete classes.
     *
     * @param element
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(Element element) {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(element);
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the Element <code>element</code>.
     * This method is a template method for concrete classes.
     *
     * @param element
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(Element element) {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(element);
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the Element <code>element</code>.
     * This method is a template method for concrete classes.
     *
     * @param element
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(Element element) {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(element);
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the Element <code>element</code>.
     * This method is a template method for concrete classes.
     *
     * @param element
     * @return Label
     */
    public Label createLabel(Element element) {
        Label label_ = createLabel();
        label_.setup(element);
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the Element <code>element</code>.
     * This method is a template method for concrete classes.
     *
     * @param element
     * @return LabelType1
     */
    public LabelType1 createLabelType1(Element element) {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(element);
        return (labelType1_);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the Stack <code>stack</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     * This method is a template method for concrete classes.
     *
     * @param stack
     * @return StartKeyboard
     */
    public StartKeyboard createStartKeyboard(RStack stack) {
        StartKeyboard startKeyboard_ = createStartKeyboard();
        startKeyboard_.setup(stack);
        return (startKeyboard_);
    }

    /**
     * Creates a <code>StartKeyboardKey</code> by the Stack <code>stack</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     * This method is a template method for concrete classes.
     *
     * @param stack
     * @return StartKeyboardKey
     */
    public StartKeyboardKey createStartKeyboardKey(RStack stack) {
        StartKeyboardKey startKeyboardKey_ = createStartKeyboardKey();
        startKeyboardKey_.setup(stack);
        return (startKeyboardKey_);
    }

    /**
     * Creates a <code>SizeSize1</code> by the Stack <code>stack</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     * This method is a template method for concrete classes.
     *
     * @param stack
     * @return SizeSize1
     */
    public SizeSize1 createSizeSize1(RStack stack) {
        SizeSize1 sizeSize1_ = createSizeSize1();
        sizeSize1_.setup(stack);
        return (sizeSize1_);
    }

    /**
     * Creates a <code>Label</code> by the Stack <code>stack</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     * This method is a template method for concrete classes.
     *
     * @param stack
     * @return Label
     */
    public Label createLabel(RStack stack) {
        Label label_ = createLabel();
        label_.setup(stack);
        return (label_);
    }

    /**
     * Creates a <code>LabelType1</code> by the Stack <code>stack</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     * This method is a template method for concrete classes.
     *
     * @param stack
     * @return LabelType1
     */
    public LabelType1 createLabelType1(RStack stack) {
        LabelType1 labelType1_ = createLabelType1();
        labelType1_.setup(stack);
        return (labelType1_);
    }

    /**
     * Creates a <code>SizeSize2</code> by the Stack <code>stack</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     * This method is a template method for concrete classes.
     *
     * @param stack
     * @return SizeSize2
     */
    public SizeSize2 createSizeSize2(RStack stack) {
        SizeSize2 sizeSize2_ = createSizeSize2();
        sizeSize2_.setup(stack);
        return (sizeSize2_);
    }

    /**
     * Creates a <code>LabelType2</code> by the Stack <code>stack</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     * This method is a template method for concrete classes.
     *
     * @param stack
     * @return LabelType2
     */
    public LabelType2 createLabelType2(RStack stack) {
        LabelType2 labelType2_ = createLabelType2();
        labelType2_.setup(stack);
        return (labelType2_);
    }

    /**
     * Creates a <code>SizeSize2</code> by the object <code>source</code>.
     * This method is a template method for concrete classes.
     *
     * @param source
     * @return SizeSize2
     */
    public SizeSize2 createSizeSize2(SizeSize2 source) {
        SizeSize2 sizeSize2_ = createSizeSize2();
        sizeSize2_.setup(source);
        return (sizeSize2_);
    }

    /**
     * Creates a <code>LabelType2</code> by the object <code>source</code>.
     * This method is a template method for concrete classes.
     *
     * @param source
     * @return LabelType2
     */
    public LabelType2 createLabelType2(LabelType2 source) {
        LabelType2 labelType2_ = createLabelType2();
        labelType2_.setup(source);
        return (labelType2_);
    }

    /**
     * Creates a default <code>RString</code>.
     * This method is a hook method of the AbstractKeyboardFactory.
     *
     * @return RString
     */
    public RString createRString() {
        return (new RString());
    }

    /**
     * Creates a <code>RString</code> by the Stack <code>stack</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     * This method is a template method for concrete classes.
     *
     * @param stack
     * @return RString
     */
    public RString createRString(RStack stack) {
        return (new RString(stack));
    }
}
