/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.boat;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>StartKeyboard</b> is generated from keyboard.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 *       </optional>
 *       </oneOrMore>
 *     </element>
 *   </zeroOrMore>
 * </element>
 * -->
 * <!-- for javadoc -->
 * <pre>       &lt;/optional&gt;
 *       &lt;/oneOrMore&gt;
 *     &lt;/element&gt;
 *   &lt;/zeroOrMore&gt;
 * &lt;/element&gt;
 * </pre>
 *
 * @version keyboard.rng (Mon Jan 26 11:51:55 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class StartKeyboard implements java.io.Serializable, Cloneable {
    // List<StartKeyboardKey>
    private java.util.List startKeyboardKey_ = new java.util.ArrayList();
    private java.util.Map xmlPIMap_;

    /**
     * Creates a <code>StartKeyboard</code>.
     *
     */
    public StartKeyboard() {
    }

    /**
     * Creates a <code>StartKeyboard</code>.
     *
     * @param source
     */
    public StartKeyboard(StartKeyboard source) {
        setup(source);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public StartKeyboard(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public StartKeyboard(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>StartKeyboard</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public StartKeyboard(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public StartKeyboard(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>StartKeyboard</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public StartKeyboard(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public StartKeyboard(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public StartKeyboard(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public StartKeyboard(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>StartKeyboard</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public StartKeyboard(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the StartKeyboard <code>source</code>.
     *
     * @param source
     */
    public void setup(StartKeyboard source) {
        int size;
        this.startKeyboardKey_.clear();
        size = source.startKeyboardKey_.size();
        for (int i = 0;i < size;i++) {
            addStartKeyboardKey((StartKeyboardKey)source.getStartKeyboardKey(i).clone());
        }
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        IKeyboardFactory factory = KeyboardFactory.getFactory();
        RStack stack = new RStack(element);
        startKeyboardKey_.clear();
        while (true) {
            if (StartKeyboardKey.isMatch(stack)) {
                addStartKeyboardKey(factory.createStartKeyboardKey(stack));
            } else {
                break;
            }
        }
        xmlPIMap_ = stack.getPIMap();
    }

    /**
     * @return Object
     */
    public Object clone() {
        IKeyboardFactory factory = KeyboardFactory.getFactory();
        return (factory.createStartKeyboard(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("keyboard");
        int size;
        size = this.startKeyboardKey_.size();
        for (int i = 0;i < size;i++) {
            StartKeyboardKey value = (StartKeyboardKey)this.startKeyboardKey_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>StartKeyboard</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>StartKeyboard</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the StartKeyboardKey property <b>startKeyboardKey</b>.
     *
     * @return StartKeyboardKey[]
     */
    public final StartKeyboardKey[] getStartKeyboardKey() {
        StartKeyboardKey[] array = new StartKeyboardKey[startKeyboardKey_.size()];
        return ((StartKeyboardKey[])startKeyboardKey_.toArray(array));
    }

    /**
     * Sets the StartKeyboardKey property <b>startKeyboardKey</b>.
     *
     * @param startKeyboardKey
     */
    public final void setStartKeyboardKey(StartKeyboardKey[] startKeyboardKey) {
        this.startKeyboardKey_.clear();
        for (int i = 0;i < startKeyboardKey.length;i++) {
            addStartKeyboardKey(startKeyboardKey[i]);
        }
    }

    /**
     * Sets the StartKeyboardKey property <b>startKeyboardKey</b>.
     *
     * @param startKeyboardKey
     */
    public final void setStartKeyboardKey(StartKeyboardKey startKeyboardKey) {
        this.startKeyboardKey_.clear();
        addStartKeyboardKey(startKeyboardKey);
    }

    /**
     * Adds the StartKeyboardKey property <b>startKeyboardKey</b>.
     *
     * @param startKeyboardKey
     */
    public final void addStartKeyboardKey(StartKeyboardKey startKeyboardKey) {
        this.startKeyboardKey_.add(startKeyboardKey);
    }

    /**
     * Adds the StartKeyboardKey property <b>startKeyboardKey</b>.
     *
     * @param startKeyboardKey
     */
    public final void addStartKeyboardKey(StartKeyboardKey[] startKeyboardKey) {
        for (int i = 0;i < startKeyboardKey.length;i++) {
            addStartKeyboardKey(startKeyboardKey[i]);
        }
    }

    /**
     * Gets number of the StartKeyboardKey property <b>startKeyboardKey</b>.
     *
     * @return int
     */
    public final int sizeStartKeyboardKey() {
        return (startKeyboardKey_.size());
    }

    /**
     * Gets the StartKeyboardKey property <b>startKeyboardKey</b> by index.
     *
     * @param index
     * @return StartKeyboardKey
     */
    public final StartKeyboardKey getStartKeyboardKey(int index) {
        return ((StartKeyboardKey)startKeyboardKey_.get(index));
    }

    /**
     * Sets the StartKeyboardKey property <b>startKeyboardKey</b> by index.
     *
     * @param index
     * @param startKeyboardKey
     */
    public final void setStartKeyboardKey(int index, StartKeyboardKey startKeyboardKey) {
        this.startKeyboardKey_.set(index, startKeyboardKey);
    }

    /**
     * Adds the StartKeyboardKey property <b>startKeyboardKey</b> by index.
     *
     * @param index
     * @param startKeyboardKey
     */
    public final void addStartKeyboardKey(int index, StartKeyboardKey startKeyboardKey) {
        this.startKeyboardKey_.add(index, startKeyboardKey);
    }

    /**
     * Remove the StartKeyboardKey property <b>startKeyboardKey</b> by index.
     *
     * @param index
     */
    public final void removeStartKeyboardKey(int index) {
        this.startKeyboardKey_.remove(index);
    }

    /**
     * Remove the StartKeyboardKey property <b>startKeyboardKey</b> by object.
     *
     * @param startKeyboardKey
     */
    public final void removeStartKeyboardKey(StartKeyboardKey startKeyboardKey) {
        this.startKeyboardKey_.remove(startKeyboardKey);
    }

    /**
     * Clear the StartKeyboardKey property <b>startKeyboardKey</b>.
     *
     */
    public final void clearStartKeyboardKey() {
        this.startKeyboardKey_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<keyboard");
        buffer.append(">");
        size = this.startKeyboardKey_.size();
        for (int i = 0;i < size;i++) {
            StartKeyboardKey value = (StartKeyboardKey)this.startKeyboardKey_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</keyboard>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<keyboard");
        buffer.write(">");
        size = this.startKeyboardKey_.size();
        for (int i = 0;i < size;i++) {
            StartKeyboardKey value = (StartKeyboardKey)this.startKeyboardKey_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</keyboard>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<keyboard");
        buffer.print(">");
        size = this.startKeyboardKey_.size();
        for (int i = 0;i < size;i++) {
            StartKeyboardKey value = (StartKeyboardKey)this.startKeyboardKey_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</keyboard>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Gets PI by name.
     *
     * @param name
     * @return String
     */
    public String rGetPi(String name) {
        return ((String)xmlPIMap_.get(name));
    }

    /**
     * Gets PI map.
     *
     * @return java.util.Map
     */
    public java.util.Map rGetPiMap() {
        return (java.util.Collections.unmodifiableMap(xmlPIMap_));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>StartKeyboard</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "keyboard")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        while (true) {
            if (!StartKeyboardKey.isMatchHungry(target)) {
                break;
            }
            $match$ = true;
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>StartKeyboard</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>StartKeyboard</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
